#!/bin/bash

#Tool to create a tarball of SPHYNX source code (ignoring data from simulations)
#It includes also the initialmodels/ directory. Comment if needed.

cd ..
file='sphynx_1.5.3'
DATE=`date +%Y-%m-%d`
file=$file'_'$DATE'.tar'
file2=$file'.bz2'
simulation='evrard/*.f90' #Modify the name of the simulation folder if needed

echo $DATE

if [ ! -f "$file" ] && [ ! -f "$file2" ]
then    
    echo 'Adding initial models:'
    tar cvf $file initialmodels 
    echo
    echo 'Adding compile folder:'
    tar rvf $file compile/makefile
    echo
    echo 'Adding sphynx folder:'
    tar rvf $file sphynx/*.f90
    tar rvf $file sphynx/*.f
    tar rvf $file sphynx/*.dat
    echo
    echo 'Adding simulation folder:'
    tar rvf $file $simulation
    echo
    echo 'Adding tools folder:'
    tar rvf $file tools/*.sh
    echo
    echo 'Compressing'
    bzip2 $file
    echo 'Checking integrity'
    bzip2 -t $file2
    checksum="$(md5sum $file2 | awk '{print $1}')"
    echo 'Checksum: '$checksum
    echo 'File '$file2' created.'
    echo 'Done!'
else
    echo "$0: File '${file}' or '${file}.bz2' already exists. Stopping."
fi

